@@ -19,7 +19,6 @@ from typing import Union
 from iamsystem.fuzzy.abbreviations import Abbreviations
 from iamsystem.fuzzy.api import FuzzyAlgo
 from iamsystem.fuzzy.api import INormLabelAlgo
-from iamsystem.fuzzy.api import ISynsProvider
 from iamsystem.fuzzy.api import SynAlgos
 from iamsystem.fuzzy.cache import CacheFuzzyAlgos
 from iamsystem.fuzzy.exact import ExactMatch
@@ -33,25 +32,24 @@ from iamsystem.keywords.api import IStoreKeywords
 from iamsystem.keywords.collection import Terminology
 from iamsystem.keywords.keywords import Keyword
 from iamsystem.keywords.util import get_unigrams
-from iamsystem.matcher.annotation import Annotation
-from iamsystem.matcher.annotation import create_annot
 from iamsystem.matcher.annotation import rm_nested_annots
-from iamsystem.matcher.annotation import sort_annot
+from iamsystem.matcher.api import IAnnotation
 from iamsystem.matcher.api import IMatcher
-from iamsystem.matcher.util import IState
-from iamsystem.matcher.util import StartState
-from iamsystem.matcher.util import TransitionState
+from iamsystem.matcher.api import IMatchingStrategy
+from iamsystem.matcher.strategy import EMatchingStrategy
+from iamsystem.matcher.strategy import WindowMatching
+from iamsystem.matcher.util import StateTransition
 from iamsystem.stopwords.api import ISimpleStopwords
 from iamsystem.stopwords.api import IStopwords
 from iamsystem.stopwords.api import IStoreStopwords
 from iamsystem.stopwords.negative import NegativeStopwords
+from iamsystem.stopwords.negative import is_a_w_2_keep_fuzzy_closure
 from iamsystem.stopwords.simple import NoStopwords
 from iamsystem.stopwords.simple import Stopwords
 from iamsystem.tokenization.api import ITokenizer
 from iamsystem.tokenization.api import TokenT
 from iamsystem.tokenization.tokenize import french_tokenizer
 from iamsystem.tokenization.tokenize import tokenize_and_order_decorator
-from iamsystem.tree.nodes import EMPTY_NODE
 from iamsystem.tree.nodes import INode
 from iamsystem.tree.trie import Trie
 
@@ -81,10 +79,22 @@ class Matcher(IMatcher[TokenT]):
         self._trie: Trie = Trie()
         self._termino: IStoreKeywords = Terminology()
         self._remove_nested_annots = True
-        if stopwords is not None:
-            self._stopwords = stopwords
-        else:
-            self._stopwords = Stopwords()
+        self._stopwords = stopwords or Stopwords()
+        self._strategy: IMatchingStrategy = WindowMatching()
+
+    @property
+    def strategy(self) -> IMatchingStrategy[TokenT]:
+        """Return the matching strategy."""
+        return self._strategy
+
+    @strategy.setter
+    def strategy(self, strategy: IMatchingStrategy) -> None:
+        """Change the matching strategy.
+
+        :param strategy: an IAMsystem matching strategy.
+        :return: None.
+        """
+        self._strategy = strategy
 
     @property
     def stopwords(self) -> IStopwords[TokenT]:
@@ -188,9 +198,7 @@ class Matcher(IMatcher[TokenT]):
         """
         return self._tokenizer.tokenize(text=text)
 
-    def add_keywords(
-        self, keywords: Iterable[Union[str, IKeyword, Dict[Any]]]
-    ) -> None:
+    def add_keywords(self, keywords: Iterable[Union[str, IKeyword]]) -> None:
         """Utility function to add multiple keywords.
 
         :param keywords: an iterable of string (labels) or
@@ -262,42 +270,45 @@ class Matcher(IMatcher[TokenT]):
         self,
         tokens: Sequence[TokenT],
         token: TokenT,
-        w_states: List[List[IState]],
-    ) -> Iterable[SynAlgos]:
+        transitions: Iterable[StateTransition],
+    ) -> List[SynAlgos]:
         """Get synonyms of a token with configured fuzzy algorithms.
 
         :param tokens: document's tokens.
         :param token: the token for which synonyms are expected.
-        :param w_states: algorithm's states.
+        :param transitions: algorithm's states.
         :return: tuples of synonyms and fuzzy algorithm's names.
         """
         syns_collector = defaultdict(list)
         for algo in self.fuzzy_algos:
             for syn, algo_name in algo.get_synonyms(
-                tokens=tokens, token=token, w_states=w_states
+                tokens=tokens, token=token, transitions=transitions
             ):
                 syns_collector[syn].append(algo_name)
         synonyms: List[SynAlgos] = list(syns_collector.items())
         return synonyms
 
-    def annot_text(self, text: str) -> List[Annotation[TokenT]]:
+    def annot_text(self, text: str) -> List[IAnnotation[TokenT]]:
         """Annotate a document.
 
         :param text: the document to annotate.
         :return: a list of :class:`~iamsystem.Annotation`.
         """
         tokens: Sequence[TokenT] = self.tokenize(text)
-        return self.annot_tokens(tokens=tokens)
+        annots = self.annot_tokens(tokens=tokens)
+        for annot in annots:
+            annot.text = text
+        return annots
 
     def annot_tokens(
         self, tokens: Sequence[TokenT]
-    ) -> List[Annotation[TokenT]]:
+    ) -> List[IAnnotation[TokenT]]:
         """Annotate a sequence of tokens.
 
         :param tokens: an ordered or unordered sequence of tokens.
         :return: a list of :class:`~iamsystem.Annotation`.
         """
-        annots = detect(
+        annots = self._strategy.detect(
             tokens=tokens,
             w=self.w,
             initial_state=self.get_initial_state(),
@@ -318,12 +329,13 @@ class Matcher(IMatcher[TokenT]):
         order_tokens=False,
         negative=False,
         remove_nested_annots=True,
+        strategy: Union[str, EMatchingStrategy] = EMatchingStrategy.WINDOW,
         string_distance_ignored_w: Optional[Iterable[str]] = None,
         abbreviations: Optional[Iterable[Tuple[str, str]]] = None,
-        spellwise: Optional[List[Dict[Any]]] = None,
-        simstring: Optional[List[Dict[Any]]] = None,
-        normalizers: Optional[List[Dict[Any]]] = None,
-        fuzzy_regex: Optional[List[Dict[Any]]] = None,
+        spellwise: Optional[List[Dict[Any, Any]]] = None,
+        simstring: Optional[List[Dict[Any, Any]]] = None,
+        normalizers: Optional[List[Dict[Any, Any]]] = None,
+        fuzzy_regex: Optional[List[Dict[Any, Any]]] = None,
     ) -> Matcher[TokenT]:
         """
         Create an IAMsystem matcher to annotate documents.
@@ -345,6 +357,9 @@ class Matcher(IMatcher[TokenT]):
             removed from keywords' tokens and so still be stopwords.
         :param remove_nested_annots: if two annotations overlap,
             remove the shorter one. Default to True.
+        :param strategy: an IAMsystem matching strategy responsible for
+            searching keywords in document.
+            Default to :class:`~iamsystem.WindowMatching`.
         :param string_distance_ignored_w: words ignored by string distance
             algorithms to avoid false positives matched.
         :param abbreviations: an iterable of tuples (short_form, long_form).
@@ -371,18 +386,23 @@ class Matcher(IMatcher[TokenT]):
             )
 
         # Start building and configuring the matcher
-        matcher = Matcher(tokenizer=tokenizer)
+        matcher: Matcher[TokenT] = Matcher(tokenizer=tokenizer)
 
         # Configure stopwords
         if isinstance(stopwords, Iterable):
             matcher.add_stopwords(words=stopwords)
         elif isinstance(stopwords, IStopwords):
             matcher.stopwords = stopwords
+        first_stopwords_instance = matcher.stopwords
 
         # Configure annot_text function
         matcher.w = w
         matcher.remove_nested_annots = remove_nested_annots
 
+        if isinstance(strategy, str):
+            strategy = EMatchingStrategy[strategy.upper()]
+        matcher.strategy = strategy.value
+
         # Add the keywords
         matcher.add_keywords(keywords=keywords)
 
@@ -394,7 +414,6 @@ class Matcher(IMatcher[TokenT]):
             )
 
         # fuzzy algorithms parameterization
-
         def _add_algo_in_cache_closure(
             cache: CacheFuzzyAlgos, matcher: Matcher
         ):
@@ -408,14 +427,14 @@ class Matcher(IMatcher[TokenT]):
 
             return add_algo_in_cache
 
-        cache = CacheFuzzyAlgos()
+        cache: CacheFuzzyAlgos = CacheFuzzyAlgos()
         add_algo_in_cache = _add_algo_in_cache_closure(
             cache=cache, matcher=matcher
         )
 
         # Abbreviations
         if abbreviations is not None:
-            _abbreviations = Abbreviations(name="abbs")
+            _abbreviations: Abbreviations[TokenT] = Abbreviations(name="abbs")
             matcher.add_fuzzy_algo(fuzzy_algo=_abbreviations)
             for abb in abbreviations:
                 short_form, long_form = abb
@@ -439,14 +458,6 @@ class Matcher(IMatcher[TokenT]):
             for params in fuzzy_regex:
                 fuzzy = FuzzyRegex(**params)
                 add_algo_in_cache(algo=fuzzy)
-                if negative:
-                    negative_stopwords = typing.cast(
-                        NegativeStopwords,
-                        matcher.stopwords,
-                    )
-                    negative_stopwords.add_fun_is_a_word_to_keep(
-                        fuzzy.token_matches_pattern
-                    )
 
         # String Distances
         # words ignored by string distance algorithms
@@ -460,9 +471,9 @@ class Matcher(IMatcher[TokenT]):
                 # don't override user's 'words2ignore':
                 if "words2ignore" not in params:
                     params["words2ignore"] = words2ignore
-                spellwise = SpellWiseWrapper(**params)
-                spellwise.add_words(words=matcher.get_keywords_unigrams())
-                add_algo_in_cache(algo=spellwise)
+                algo = SpellWiseWrapper(**params)
+                algo.add_words(words=matcher.get_keywords_unigrams())
+                add_algo_in_cache(algo=algo)
 
         # Parameterize simstring
         if simstring is not None:
@@ -476,70 +487,18 @@ class Matcher(IMatcher[TokenT]):
                 )
                 add_algo_in_cache(algo=ss_algo)
 
+        # if negative stopwords is used:
+        # add a function that calls fuzzy algorithms to check if any
+        # synonym is returned by them.
+        # https://github.com/scossin/iamsystem_python/issues/15
+        if negative:
+            is_a_w_2_keep_fuzzy = is_a_w_2_keep_fuzzy_closure(
+                fuzzy_algos=matcher.fuzzy_algos,
+                stopwords=first_stopwords_instance,
+            )
+            negative_stopwords = typing.cast(
+                NegativeStopwords,
+                matcher.stopwords,
+            )
+            negative_stopwords.add_fun_is_a_word_to_keep(is_a_w_2_keep_fuzzy)
         return matcher
-
-
-def detect(
-    tokens: Sequence[TokenT],
-    w: int,
-    initial_state: INode,
-    syns_provider: ISynsProvider,
-    stopwords: IStopwords,
-) -> List[Annotation[TokenT]]:
-    """Main internal function that implements iamsystem's algorithm.
-    Algorithm formalized in https://ceur-ws.org/Vol-3202/livingner-paper11.pdf
-
-    :param tokens: a sequence of :class:`~iamsystem.IToken`.
-    :param w: window, how many previous tokens can the algorithm look at.
-    :param initial_state: a node/state in the trie, i.e. the root node.
-    :param syns_provider: a class that provides synonyms for each token.
-    :param stopwords: an instance of :class:`~iamsystem.IStopwords`
-    that checks if a token is a stopword.
-    :return: A list of :class:`~iamsystem.Annotation`.
-    """
-    annots: List[Annotation] = []
-    # +1 to insert the start_state.
-    w_states: List[List[IState]] = [[]] * (w + 1)
-    start_state = StartState(node=initial_state)
-    # [w] element stores only the start_state. This element is not replaced.
-    w_states[w] = [start_state]
-    # different from i for a stopword-independent window size.
-    count_not_stopword = 0
-    stop_tokens: List[TokenT] = []
-    for i, token in enumerate(tokens):
-        if stopwords.is_token_a_stopword(token):
-            stop_tokens.append(token)
-            continue
-        count_not_stopword += 1
-        syns_algos: Iterable[SynAlgos] = syns_provider.get_synonyms(
-            tokens=tokens, token=token, w_states=w_states
-        )
-        # stores matches between document's tokens and keywords'tokens.
-        tokens_states: List[TransitionState] = []
-
-        # 1 to many synonyms depending on fuzzy_algos configuration.
-        for syn, algos in syns_algos:
-            # 0 to many states for [0] to [w-1] ; [w] only the start state.
-            for states in w_states:
-                for state in states:
-                    new_state = state.node.jump_to_node(syn)
-                    # when no path is found, EMPTY_NODE is returned.
-                    if new_state is EMPTY_NODE:
-                        continue
-                    token_state = TransitionState(
-                        parent=state, node=new_state, token=token, algos=algos
-                    )
-                    tokens_states.append(token_state)
-                    if new_state.is_a_final_state():
-                        annot = create_annot(
-                            last_el=token_state, stop_tokens=stop_tokens
-                        )
-                        annots.append(annot)
-        # function 'count_not_stopword % w' has range [0 ; w-1]
-        w_states[count_not_stopword % w].clear()
-        w_states[count_not_stopword % w] = tokens_states
-        # Mypy: Incompatible types in assignment (expression has type
-        # "List[TokenState[Any]]", target has type "List[State]")
-        # but TokenState is a sublcass of State.
-    sort_annot(annots)  # mutate the list like annots.sort()
-    return annots
